<?php
/*
* Modifié par Yannick KERMAREC
* http://www.kermarec.com
*
* 2007-2011 PrestaShop 
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2011 PrestaShop SA
*  @version  Release: $Revision: 6594 $
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

define('DEBUG', 1);

include_once('../../config/config.inc.php');
include_once('mondialrelay.php');

if (Tools::getValue('secure_key') != Configuration::get('MONDIAL_RELAY_SECURE_KEY'))
	exit;

$expeditions = Db::getInstance()->ExecuteS('
SELECT ms.`exp_number`, ms.`id_cart`, o.`id_order`
FROM `'._DB_PREFIX_.'mr_selected` ms
LEFT JOIN `'._DB_PREFIX_.'orders` o ON (o.`id_cart` = ms.`id_cart`) 
WHERE `exp_number` != 0 ORDER BY o.`id_order` DESC');

if (empty($expeditions))
	exit;

$params = array(
'Enseigne' => Configuration::get('MR_ENSEIGNE_WEBSERVICE'),
'Langue' => 'FR'
);

require_once('kit_mondialrelay/tools/nusoap/lib/nusoap.php');
$client_mr = new nusoap_client("http://www.mondialrelay.fr/webservice/Web_Services.asmx?WSDL", true);
$client_mr->soap_defencoding = 'UTF-8';
$client_mr->decode_utf8 = false;

foreach ($expeditions as $expedition)
{
	if ($expedition['id_order'] == NULL)
		continue;
  
  $params['Expedition'] = $expedition['exp_number'];
	$params['Security'] = strtoupper(md5($params['Enseigne'].$params['Expedition'].'FR'.Configuration::get('MR_KEY_WEBSERVICE')));
	
	$is_delivered = 0;
	$result_mr = $client_mr->call('WSI2_TracingColisDetaille', $params, 'http://www.mondialrelay.fr/webservice/', 'http://www.mondialrelay.fr/webservice/WSI2_TracingColisDetaille');
  updateOrderHistory ($result_mr['WSI2_TracingColisDetailleResult']['Tracing']['ret_WSI2_sub_TracingColisDetaille'], $expedition['id_order']);
}


function updateOrderHistory ($History, $id_order) {
  if ($History && $id_order) {
    /* DÉCLARATION DES STAUTS MR ET PS */
    $PSHitory = array(); # Tableau d'historique à ajouter
    $STEPS_CANCELED = array(6,7); # Liste des stauts PS à ne pas mettre à jour. Evite de mettre à jour les statuts des ventes annulées ou remboursés par exemple.
    /* TABLEAUX PERMETTANT DE FAIRE LE LIEN ENTRE LES STATUTS MR ET PS */
    $STEPS_MR_PS = array(
      'PRÉPARATION EN COURS'        => 3,
      'RÉCEPTION DES DONNÉES'       => 27,
      'PRISE EN CHARGE EN AGENCE'   => 4,
      'DISPONIBLE AU POINT RELAIS'  => 16,
      'COLIS LIVRÉ'                 => 5,
      'RETOUR DU COLIS'             => 20
    );
    $STEPS_ORDER = array(); # Stoquera les statuts PS avec leur poids respectifs. Pour déclarer l'ordre des statuts souhaité
    $i = 0; foreach ($STEPS_MR_PS as $val) { $i++; $STEPS_ORDER[$val] = $i; }
    $STEPS_PS_MR = array_flip($STEPS_MR_PS);

    $History = array_reverse ( $History ); # Inversion de l'ordre des statuts MR pour lire à l'envers
    $LastStep = OrderHistory::getLastOrderState((int)($id_order))->id; # Recherche du dernier statut PS
    if (!in_array($LastStep, $STEPS_CANCELED)) # Test si le dernier statut fait partie des statuts à éviter table $STEPS_CANCELED
    /* PARCOURS DES HISTORIQUE MR DE BAS EN HAUT (car inversion du tableau en début de fonction) */
    foreach ($History as $result) {
      if ($result['Libelle'] == '') continue; # Saute les libéllés vide de MR
      if (OrderStateExist ($id_order, $STEPS_MR_PS[$result['Libelle']])) break; # Arrête le parcours dès qu'on arrive à une étape MR déjà référencée dans PS
      if($STEPS_ORDER[$STEPS_MR_PS[$result['Libelle']]] > $STEPS_ORDER[$LastStep]) # Vérifie que l'étape à ajouter est bien une étape suivant la dernière étape enregistrée dans PS
        array_push($PSHitory, $STEPS_MR_PS[$result['Libelle']]); # Ajoute l'étape MR avec le code PS dans le tableau qui sera traité plus bas
    }
    $PSHitory = array_reverse ($PSHitory); # Remet les statuts enregistrés dans le bon ordre
    /* CRÉATION DES STATUTS NON ENREGISTRÉS DANS PS */
    foreach ($PSHitory as $Step) {
      $history = new OrderHistory();
		  $history->id_order = (int)($id_order);
		  $history->changeIdOrderState((int)($Step), (int)($id_order)); # Ajoute le statut dans PS
		  $history->addWithemail(); # Envoi l'email s'il y en a un
    }
    debug ($PSHitory, $History, $id_order, $STEPS_PS_MR, $LastStep); # Affiche des infos si le script est executé dans un navigateur afin de vérifier ce qu'il a modifié.
  }
}

function OrderStateExist ($id_order, $id_order_state) {
  /* FONCTION QUI TEST LA BASE PS POUR SAVOIR SI UN STATUT À DÉJÀ ÉTÉ ENREGISTRÉ POUR UNE COMMANDE DONNÉE */
  $id_order_state = Db::getInstance()->getValue('
		SELECT *
		FROM `'._DB_PREFIX_.'order_history`
		WHERE `id_order` = '.(int)($id_order).'
		AND `id_order_state` = '.(int)($id_order_state).'
		ORDER BY `date_add` DESC, `id_order_history` DESC');
		if (!$id_order_state)
			return false;
		return true;
}

function debug ($PSHitory, $History, $id_order, $STEPS_PS_MR, $LastStep) {
  if ( DEBUG && (count($PSHitory) > 0)) {
    $History = array_reverse ( $History );
    foreach ($History as $result) { if ($result['Libelle'] == '') continue; print $id_order." : ".$result['Libelle']."<br>"; }
    print $id_order." : Statut actuel : ".$STEPS_PS_MR[$LastStep]." (".$LastStep.") poids : ".$STEPS_ORDER[$LastStep]."<br>";
    print "<span style='background-color:red'>";
    print $id_order." : ";
    print_r ($PSHitory);
    print "</span><br><br>";
  }
}
